#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.0
//#include "FWHM"

Constant hc=12.39841857			// h*c (keV-)

Menu "Macros"
	"single slit pattern",MakeDiffractionGraph()
End




Function ReCalc()
//	NVAR fl=fl, aperture=aperture, lambda=lambda
	Variable fl = NumVarOrDefault("fl",NaN)/1e3
	Variable aperture = NumVarOrDefault("aperture",NaN)/1e6
	Variable lambda = NumVarOrDefault("lambda",NaN)*1e-10
	Make/N=401/O fn,fn2
	Variable range = fl * asin(3/aperture*lambda)		// meters
	SetScale/I x -range  ,range  ,"m", fn,fn2
	fn = intens(x)
	fn2 = doublet(x)
End

Function intens(dist)
	Variable dist
//	NVAR fl=fl, aperture=aperture, lambda=lambda
	Variable fl = NumVarOrDefault("fl",NaN)/1e3
	Variable aperture = NumVarOrDefault("aperture",NaN)/1e6
	Variable lambda = NumVarOrDefault("lambda",NaN)*1e-10
	Variable beta = PI*aperture/lambda*sin(dist/fl)
	return (sin(beta)/beta)^2
End

Function doublet(dist)
	Variable dist
//	NVAR fl=fl, aperture=aperture, lambda=lambda
	Variable fl = NumVarOrDefault("fl",NaN)/1e3
	Variable aperture = NumVarOrDefault("aperture",NaN)/1e6
	Variable lambda = NumVarOrDefault("lambda",NaN)*1e-10
	Variable beta = PI*aperture/lambda*sin(dist/fl)
	return (sin(beta)/beta)^2 + (sin(beta+PI)/(beta+PI))^2
End


Function MakeDiffractionGraph()
	initSingleSlit()
	ReCalc()
	Display /W=(5,44,773,567)/K=1 fn2,fn
	ModifyGraph gfMult=130
	ModifyGraph lSize(fn)=2,rgb(fn)=(0,12800,52224),tick=2,mirror=1,minor=1,lowTrip=0.001
	ModifyGraph standoff=0,axOffset(left)=-1.5,axOffset(bottom)=-0.444444, lstyle(fn2)=1
	Label left "normalized Intensity"
	Label bottom "position  (\\U)"
	TextBox/N=text0/F=0/S=3/X=4.68/Y=5.53 "\\f06Single Slit Diffraction Pattern\\f00\rFWHM = \\{\"%.0W1Pm\",callFWHMofWave(fn)}"
	AppendText "focal length = \\{fl} mm\raperture = \\{aperture} m\rEnergy = \\{keV} keV"
	SetVariable setFocalLength,pos={90,60},size={150,23},proc=SetAllVarProc,title="fl (mm)"
	SetVariable setFocalLength,fSize=14,format=""
	SetVariable setFocalLength,limits={0,inf,5},value= fl
	SetVariable setAperture,pos={90,85},size={150,23},proc=SetAllVarProc,title="Size (m)"
	SetVariable setAperture,fSize=14,format=""
	SetVariable setAperture,limits={0,inf,5},value= aperture
	SetVariable setEnergy,pos={90,35},size={150,23},proc=SetAllVarProc,title="E (keV)"
	SetVariable setEnergy,fSize=14,format="",limits={0,inf,1},value= keV
	PopupMenu presets,pos={90,110},size={103,20},proc=PresetMenuProc
	PopupMenu presets,mode=1,popvalue="pick a preset",value= #"\"pick a preset;Tinsley Mirror;2nd nanoMirror;long micro-mirror;short micro-mirror;1st Japanese;2nd Japanese\""
End
Function callFWHMofWave(ww)
	Wave ww
	return FWHM_peak(ww)
End
Static Function FWHM_peak(ww)
	Wave ww
	WaveStats/Q ww
	Make/N=2/O/D ww_temp__
	FindLevels /D=ww_temp__ /N=6 /Q ww, ((V_max+V_min)/2)
	if (V_LevelsFound<2)
		return NaN					// no peak
	endif

	Variable p1,p2				// points values of the half widths
	Variable x1,x2				// x values of halfwidths
	p1 = BinarySearch(ww_temp__, V_maxloc)
	x2 = ww_temp__[p1+1]
	x1 = ww_temp__[p1]
	KillWaves/Z ww_temp__
	Variable/G V_center=(x2+x1)/2
	return abs(x2-x1)
End

Function SetAllVarProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	if (stringmatch(varName,"fl") || stringmatch(varName,"aperture"))
		PopupMenu presets,mode=1
	endif
	NVAR keV=keV
	NVAR lambda=lambda
	lambda = hc/keV
	ReCalc()
End
Function PresetMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	NVAR fl=fl, aperture=aperture, keV=keV
	strswitch(popStr)
		case "Tinsley Mirror":
			fl = 60				// mm
			aperture=120		// m
			break
		case "2nd nanoMirror":
			fl = 30				//mm
			aperture=60		// m
			break
		case "long micro-mirror":
			fl = 130			// mm
			aperture=270		// m
			break
		case "short micro-mirror":
			fl = 60				// mm
			aperture=120		// m
			break
		case "1st Japanese":
			fl = 300			// 300 mm
			aperture = 140		// m
			keV = 15
			break
		case "2nd Japanese":
			fl = 150			// 150 mm
			aperture = 140		// m
			keV = 15
			break
	endswitch
	ReCalc()
End

Function initSingleSlit()
	if (exists("fl")!=1)
		Variable/G fl = 60			// mm
	endif
	if (exists("aperture")!=1)
		Variable/G aperture = 120	// m
	endif
	if (exists("keV")!=1)
		Variable/G keV = 15
	endif
	Variable/G lambda = hc/keV
End

//Constant fl=0.3						// japanese mirrors
//Constant aperture=140e-6
//Constant lambda=0.826561e-10

//Constant fl=0.06
//Constant aperture=40e-6
//Constant lambda=0.826561e-10		// == 15 keV
